#!/bin/sh
# mkf processor.  See documentation at the end of this file.
exec awk '
{
  if ($0 ~ /^[^#]/) {		# Makes sure any keyword starts
    if (print_level == 0) print	# at the beginning of a line.
  }
  else if ($1 == "#define") {	# Input is "#define VARIABLE DATA ...".
    if (NF < 2) {
      print "Syntax error on line " NR ": " $0 | "cat 1>&2"
      parse_level = 0
      exit 1
    }
    var = $2			# Variable gets a string computed from
    $1 = $2 = ""		# the concatenation of the input data
    env[var] = $0		# separated by the output field separator.
  }
  else if ($1 == "#if") {	# Input is "#if VARIABLE PATTERN".
    if (NF > 3 || NF < 2) {
      print "Syntax error on line " NR ": " $0 | "cat 1>&2"
      parse_level = 0		# #if evaluates to true when copying is
      exit 1			# enabled, the variable has been defined,
    }				# and the pattern is a substring of the
    parse_level++		# value of the variable.
##  if (print_level > 0 || !($2 in env) || !index(env[$2], $3)) print_level++
    if (print_level > 0 || !index(env[$2], $3)) print_level++
  }
  else if ($1 == "#else") {
    if (parse_level == 0) {	# Input is "#else IGNORED ...".
      print "Syntax error on line " NR " --missing #if: " $0 | "cat 1>&2"
      parse_level = 0
      exit 1
    }				# Change sense of last #if.
    if (print_level == 0) print_level = 1
    else if (print_level == 1) print_level = 0
  }
  else if ($1 == "#endif") {	# Input is "#endif IGNORED ...".
    if (parse_level == 0) {
      print "Syntax error on line " NR " --missing #if: " $0 | "cat 1>&2"
      parse_level = 0
      exit 1
    }
    parse_level--		# Terminate matching #if.
    if (print_level > 0) print_level--
  }
  else if (print_level == 0) print
}
END {
  if (parse_level > 0) {
    print "Syntax error -- missing #endif" | "cat 1>&2"
    exit 1
  }
}
' "$@"

#	mkf -- John D. Ramsdell -- December 1990
# mkf copies text conditionally based on the values of mkf variables.
#
# A variable gets a value using the #define command.
#
#	#define VARIABLE DATA ...
#
# assigns a string to the variable which is computed by concatenating
# the zero or more data items that follow the variable, each separated
# by a single space.
#
# The decision whether to copy is based on the #if command.
#
#	#if VARIABLE PATTERN
#
# copies text if
# (1) copying was enabled before the #if,
# (2) the variable has been defined using the #define command, and
# (3) the pattern is a substring of the value of the variable.
#
# If copying was enable before an #if command, the sense of the #if can
# by changed by a text line beginning with #else.  An #if affects the
# decision to copy text until a matching #endif is found.  #if ... #endif
# commands can be nested.
